@@ -18,7 +18,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =end
 
 # Feed2Imap version
-F2I_VERSION = '0.9'
+F2I_VERSION = '0.9.2'
 
 require 'feed2imap/config'
 require 'feed2imap/cache'
@@ -102,6 +102,9 @@ class Feed2Imap
     @logger.info("Fetching and filtering feeds ...")
     ths = []
     mutex = Mutex::new
+    sparefetchers = 16 # max number of fetchers running at the same time.
+    sparefetchers_mutex = Mutex::new
+    sparefetchers_cond = ConditionVariable::new
     @config.feeds.each do |f|
       ths << Thread::new(f) do |feed|
         begin
@@ -109,6 +112,12 @@ class Feed2Imap
           lastcheck = @cache.get_last_check(feed.name) 
           if feed.needfetch(lastcheck)
             mutex.unlock
+            sparefetchers_mutex.synchronize do
+              while sparefetchers <= 0
+                sparefetchers_cond.wait(sparefetchers_mutex)
+              end
+              sparefetchers -= 1
+            end
             if feed.url
               s = HTTPFetcher::fetch(feed.url, @cache.get_last_check(feed.name))
             elsif feed.execurl
@@ -123,6 +132,10 @@ class Feed2Imap
                 s = stdout.read
               end
             end
+            sparefetchers_mutex.synchronize do
+              sparefetchers += 1
+              sparefetchers_cond.signal
+            end
             mutex.lock
             feed.body = s
             @cache.set_last_check(feed.name, Time::now)
@@ -205,7 +218,8 @@ class Feed2Imap
             f.imapaccount.updatemail(f.folder, email,
                                      i.cacheditem.index, i.date || Time::new)
           end
-          newitems.each do |i|
+          # reverse is needed to upload older items first (fixes gna#8986)
+          newitems.reverse.each do |i|
             email = item_to_mail(i, i.cacheditem.index, false, f.name)
             f.imapaccount.putmail(f.folder, email, i.date || Time::new)
           end
@@ -213,7 +227,8 @@ class Feed2Imap
       rescue
         @logger.fatal("Exception caught while uploading mail to #{f.folder}: #{$!}")
         puts $!.backtrace
-        next
+        @logger.fatal("We can't recover from IMAP errors, so we are exiting.")
+        exit(1)
       end
       begin
         @cache.commit_cache(f.name)
@@ -224,9 +239,14 @@ class Feed2Imap
     end
     @logger.info("Finished. Saving cache ...")
     begin
-      File::open(@config.cache, 'w') { |f| @cache.save(f) }
+      File::open("#{@config.cache}.new", 'w') { |f| @cache.save(f) }
     rescue
-      @logger.fatal("Exception caught while writing cache to #{@config.cache}: #{$!}")
+      @logger.fatal("Exception caught while writing new cache to #{@config.cache}.new: #{$!}")
+    end
+    begin
+      File::rename("#{@config.cache}.new", @config.cache)
+    rescue
+      @logger.fatal("Exception caught while renaming #{@config.cache}.new to #{@config.cache}: #{$!}")
     end
     @logger.info("Closing IMAP connections ...")
     @config.imap_accounts.each_value do |ac|
